#!/usr/bin/env python
# -*- coding: UTF8 -*-

# Python batch-resizer.py
# Author : Anthony Levaufre
# Licence : GPL V2

import sys
import os
import os.path
import shutil
import gtk
import Image

from math import floor

from SimpleGladeApp import SimpleGladeApp
from SimpleGladeApp import bindtextdomain

app_name = "batch-resizer"
app_version = "1.1"

glade_dir = ""
locale_dir = ""

bindtextdomain(app_name, locale_dir)

class Collec:
	def __init__(self, source, cible):
		self.source = source
		self.cible = cible
		self.status = 1

	def create_files(self,renom,prefix,suffix,numerot):
		if not os.path.isdir(self.source):
			#renvoie une erreur si le rep source n'existe pas
			print "la source n'est pas un repertoire"
			self.status = 0
			
		if not os.path.isdir(self.cible):
			#cree le rep cible s'il n'existe pas
			os.makedirs(self.cible)
		if not os.path.isdir(self.cible):
			#renvoie une erreur si le rep cible n'existe pas
			print "la cible n'est pas un repertoire"
			self.status = 0
					
		images_names = []
		self.coll = []
		for fichier in os.listdir(self.source):
			if (os.path.isfile(os.path.join(self.source,fichier)) and (os.path.splitext(fichier.lower())[1]==".jpg" or os.path.splitext(fichier.lower())[1]==".gif" or os.path.splitext(fichier.lower())[1]==".png")):
				images_names += [fichier]
		images_names.sort()
		images_new_names = []
		if renom:
			for i in range(len(images_names)):
				images_new_names += [prefix+str(numerot+i)+suffix+os.path.splitext(images_names[i])[1]]
		else:
			images_new_names = images_names
		
		index = 0
		for fichier in images_new_names:
			if not os.path.samefile(self.source,self.cible) or renom:
				#si le fichier cible n'existe pas, on le crée
				shutil.copy(os.path.join(self.source,images_names[index]),os.path.join(self.cible,fichier))
				os.chmod(os.path.join(self.cible,fichier),0644)
			img = Imag(self.cible,fichier)
			self.coll += [img]
			index+=1
		return self.status
	
	def rotation(self,type):
		for img in self.coll:
	   		while gtk.events_pending():
				gtk.main_iteration()
			img.rotate(type)

	def redimen(self, largeur, hauteur, agrandir, filter):
		for image in self.coll:
	   		while gtk.events_pending():
				gtk.main_iteration()
			image.redim(agrandir, largeur, hauteur, filter)
	
	def get_coll(self):
		for image in self.coll:
	   		while gtk.events_pending():
				gtk.main_iteration()
			print image.get_name()

class Imag:
	def __init__(self, chemin, fichier):
		self.path = chemin
		self.name = fichier

	def rotate(self, type):
		#on ouvre l'image
		im = Image.open(os.path.join(self.path,self.name))
		#on tourne
		if type == 1:
			im = im.transpose(Image.ROTATE_270)
		elif type == 2:
			im = im.transpose(Image.ROTATE_90)
		elif type == 3:
			im = im.transpose(Image.ROTATE_180)
		else:
			#cas des tags EXIF
			if os.path.splitext(self.name.lower())[1] == ".jpg":
				exif_tags=[]
				try:
					exif_tags = im._getexif()
				except:
					print "probleme de lecture du champ exif"
				if exif_tags:
					orientation = exif_tags[274]
					if orientation == 6:
						im = im.transpose(Image.ROTATE_270)
					elif orientation == 8:
						im = im.transpose(Image.ROTATE_90)
					elif orientation == 1:
						print "pas de rotation utile : ",self.name
					else:
						print "orientation non reconnue : ",self.name
				else:
					print "pas de tag exif : ",self.name
	 
		#on enregistre
		im.save(os.path.join(self.path,self.name))
	
	def redim(self, agrandir, largeur, hauteur, filter):
		if largeur:
			lmax = int(largeur)
		else:
			lmax = 1000000
		if hauteur:
			hmax = int(hauteur)
		else:
			hmax = 1000000
		
		#on ouvre l'image
		im = Image.open(os.path.join(self.path,self.name))

		#on lit la hauteur et la largeur
		self.L = im.size[0]
		self.H = im.size[1]
		self.ratio = float(self.L)/float(self.H)

		if agrandir or self.L > lmax or self.H > hmax:
			#redim
			if self.ratio >= float(lmax)/float(hmax):
				self.L = lmax
				self.H = int(floor(self.L/self.ratio))
				im = im.resize((self.L,self.H),filter)
				print "ok1 : ",self.L,self.H,filter
			else:
				self.H = hmax
				self.L = int(floor(self.H*self.ratio))
				im = im.resize((self.L,self.H),filter)
				print "ok2 : ",self.L,self.H,filter
		#on enregistre
		print "enregistrement"
		print im.size
		im.save(os.path.join(self.path,self.name))
		print "ok"
		
	def get_name(self):
		return self.name

class Nszbr(SimpleGladeApp):
	def __init__(self, path="batch-resizer.glade",root="nszbr",domain=app_name, **kwargs):
		path = os.path.join(glade_dir, path)
		SimpleGladeApp.__init__(self, path, root, domain, **kwargs)
		#selecteur de dossiers dans /home
		for i in (1,2):
			gtk_filechooser = self.get_widget("filechooserbutton"+str(i))
			gtk_filechooser.set_current_folder('/home')
		gtklabel = self.get_widget("label23")
		gtklabel.set_text("GTK Batch Resizer V"+app_version)
		
	def new(self):
		print "A new %s has been created" % self.__class__.__name__

	def on_checkbutton1_toggled(self, widget, *args):
		gtkcheck = self.get_widget("checkbutton1")
		self.on_sensitivity_toggled(gtkcheck,1)
		if gtkcheck.get_active():
			gtkrenam = self.get_widget("checkbutton2")
			gtkrenam.set_active(False)

	def on_checkbutton2_toggled(self, widget, *args):
		gtkcheck = self.get_widget("checkbutton2")
		self.on_sensitivity_toggled(gtkcheck)
		if gtkcheck.get_active():
			gtkecras = self.get_widget("checkbutton1")
			gtkecras.set_active(False)
        
	def on_checkbutton5_toggled(self, widget, *args):
		gtkcheck = self.get_widget("checkbutton5")
		self.on_sensitivity_toggled(gtkcheck)
    
	def on_checkbutton4_toggled(self, widget, *args):
		gtkcheck = self.get_widget("checkbutton4")
		self.on_sensitivity_toggled(gtkcheck)
    
	def on_sensitivity_toggled(self, togglebutton, bit=0):
		container = togglebutton.get_parent()
		children = container.get_children()
		for child in children:
            # don't disable our toggle
			if child != togglebutton:
				if bit:
					child.set_sensitive(not togglebutton.get_active())
				else:
					child.set_sensitive(togglebutton.get_active())
	def on_button15_released(self, widget, *args):
		#message d'attente
		wait_window = SimpleGladeApp("batch-resizer.glade", "processing")
		wait_widget = wait_window.get_widget("processing")
		wait_window.gtk_widget_show(wait_widget)			
	   	while gtk.events_pending():
			gtk.main_iteration()
		#rep source
		gtk_filechooser = self.get_widget("filechooserbutton1")
	   	source = gtk_filechooser.get_current_folder()
	   	
		#ecrasement
		checkbox = self.get_widget("checkbutton1")
	   	ecrase = checkbox.get_active()
    	
		#rep cible
		if ecrase:
			cible = source
		else:
			gtk_filechooser = self.get_widget("filechooserbutton2")
			cible = gtk_filechooser.get_current_folder()
		#renommage
	   	checkbox = self.get_widget("checkbutton2")
	   	renom = checkbox.get_active()
    
		#message d'erreur en cas d'incoherence des données
		if (source == cible) and (not ecrase) and (not renom):
			wait_window.gtk_widget_destroy(wait_widget)	
			alert = SimpleGladeApp("batch-resizer.glade", "dialog1")
			alertxt = "Si vous souhaitez écraser les fichiers, cliquez sur Ecraser l'original"
			alert.label11.set_text(alertxt)
			return
		
		checkbox = self.get_widget("checkbutton5")
	   	redim = checkbox.get_active()
    	
	   	checkbox = self.get_widget("checkbutton4")
	   	rotat = checkbox.get_active()
    	
	   	checkbox = self.get_widget("checkbutton3")
	   	agrand = checkbox.get_active()

	   	#champs nouveau nom
		gtkentry = self.get_widget("entry1")
		prefix = gtkentry.get_text()
		gtkentry = self.get_widget("entry2")
		suffix = gtkentry.get_text()
		gtkentry = self.get_widget("entry3")
		numerot = gtkentry.get_text()
		if numerot=="":
			numerot=0
		else:
			numerot=int(numerot)
		
		###
		#Creation de la collection d'images
		###
		collimage = Collec(source,cible)
		status = collimage.create_files(renom,prefix,suffix,numerot)
		#alerte en cas de pb de creation
		if not status:
			alert = SimpleGladeApp("batch-resizer.glade", "dialog1")
			alertxt = "Problème de création de fichiers"
			alert.label11.set_text(alertxt)
			return
		collimage.get_coll()

	   	if rotat :
	   		gtkpuce = self.get_widget("EXIF")
	   		if gtkpuce.get_active():
				rotatype = 0
	   		gtkpuce = self.get_widget("right90")
	   		if gtkpuce.get_active():
				rotatype = 1
	   		gtkpuce = self.get_widget("left90")
	   		if gtkpuce.get_active():
				rotatype = 2
	   		gtkpuce = self.get_widget("right180")
	   		if gtkpuce.get_active():
				rotatype = 3
			collimage.rotation(rotatype)
		
	   	if redim : 
	   		gtkentry = self.get_widget("entry4")
	   		largeur = gtkentry.get_text()
	   		gtkentry = self.get_widget("entry5")
	   		hauteur = gtkentry.get_text()
	   		if not largeur and not hauteur:
				alert = SimpleGladeApp("batch-resizer.glade", "dialog1")
				alertxt = "Vous devez spécifier une largeur ou une hauteur"
				alert.label11.set_text(alertxt)
				wait_window.gtk_widget_destroy(wait_widget)	
				return
										
			checkbox = self.get_widget("checkbutton3")
	   		agrandir = checkbox.get_active()
			checkbox = self.get_widget("basic")
	   		basic = checkbox.get_active()
			checkbox = self.get_widget("linear")
	   		linear = checkbox.get_active()
			checkbox = self.get_widget("cubic")
	   		cubic = checkbox.get_active()
			if basic:
				filter = Image.NEAREST
			elif linear:
				filter = Image.BILINEAR
			elif cubic:
				filter = Image.BICUBIC
			collimage.redimen(largeur,hauteur,agrandir,filter)
		'''
		print "source : "+source
	   	print "cible : "+cible
	   	print "ecrase : "+str(ecrase)
	   	print "renom : "+str(renom)
	   	print "redim : "+str(redim)
	   	print "rotat : "+str(rotat)
	   	print "agrand : "+str(agrand)
	   	print "prefix : "+prefix
	   	print "suffix : "+suffix
	   	print "numerot : "+numerot
	   	print "largeur : "+largeur
	   	print "hauteur : "+hauteur
		print "agrandir : "+str(agrandir)
	   	print "EXIF : "+str(exif)
	   	print "rigth90 : "+str(right90)
	  	print "left90 : "+str(left90)
		print "right180 : "+str(right180)
		'''
		wait_window.gtk_widget_destroy(wait_widget)	
		print "done"
#-- main {

def main():
	nszbr = Nszbr()
	nszbr.run()

if __name__ == "__main__":
	main()

#-- main }
